import os, sys, glob
v1,v2,v3 = sys.version_info[:3]
if 10*v1 + v2 < 24:
  print "Please install Python 2.4 or higher"
  sys.exit()
  
currdir = os.path.split(os.path.abspath(__file__))[0]

alltests = {}
if len(sys.argv) != 2:
  print "Syntax: python analysis.py <PDB code>"
  sys.exit()
prot = sys.argv[1]
protdir = currdir + os.sep + ".." + os.sep + prot
if not os.path.exists(protdir):
  print "Complex with PDB code %s does not exist" % prot
  sys.exit()

currdir = os.path.split(__file__)[0]
sys.path.insert(0, currdir)
testmodnames = glob.glob("test*.py")
testmods = [__import__(m[:-len(".py")]) for m in testmodnames]
for c in ("test", "chaintest", "predtest", "runtest", "docktest"):
  alltests[c] = {}
  tests = alltests[c]
  for mod in testmods:
    currtests  = [t for t in dir(mod) if t.startswith(c)]
    for t in currtests:
      number = t[len(c):]
      if number.isdigit():
        number = int(number)
        if number not in tests: tests[number] = []
        tests[number].append((m, mod))
      else:
        continue

  numbers = sorted(list(tests.keys()))
  for n in numbers:
    if len(tests[n]) > 1:
      for num in range(1,numbers[-1]+2):
        if num not in numbers: break
      raise Exception("%s%d() is defined in multiple modules:\n  %s.\nFirst free test function: %s%d()" % (c, n, ",".join([x[0] for x in tests[n]]), num,c)) 

tests = alltests["test"]
numbers = sorted(list(tests.keys()))
for n in numbers:
  ret = getattr(tests[n][0][1], "test"+str(n))(protdir, prot)
  if ret != None: print "##TEST " +str(n) + ": ", ret
print 

chainfile = protdir + os.sep + ".." + os.sep + "chains.list"
chains = [l.split()[1:3] for l in open(chainfile).readlines() if l.split()[0] == prot][0]
tests = alltests["chaintest"]
predtests = alltests["predtest"]
numbers = sorted(list(tests.keys()))
prednumbers = sorted(list(predtests.keys()))

for chain in chains:  
  for n in numbers:
    ret = getattr(tests[n][0][1], "chaintest"+str(n))(protdir, prot, chain)
    if ret != None: print "##CHAINTEST %s " % chain +str(n) + ": ", ret
  print     
  predfiles = glob.glob("../%s/predictions/%s%s.*" % (prot, prot, chain))
  predfiles.sort()
  for predfile in predfiles:
    ending = os.path.splitext(predfile)[1][1:]
    if ending in ("bins", "resdist", "sur"): continue
    pred = ending
    for n in prednumbers:
      ret = getattr(predtests[n][0][1], "predtest"+str(n))(protdir, prot, chain, predfile)
      if ret != None: print "##PREDTEST %s %s " % (chain, pred) +str(n) + ": ", ret
  print

tests = alltests["runtest"]
numbers = sorted(list(tests.keys()))
for run in ("run1", "run2", "run3"):
  rundir = os.sep.join((protdir, run))
  if not os.path.exists(rundir): continue
  for n in numbers:
    ret = getattr(tests[n][0][1], "runtest"+str(n))(protdir, prot, run)
    if ret != None: print "##RUNTEST %s " % run +str(n) + ": ", ret
  print 
print  

tests = alltests["docktest"]
numbers = sorted(list(tests.keys()))
for n in numbers:
  ret = getattr(tests[n][0][1], "docktest"+str(n))(protdir, prot)
  if ret != None: print "##DOCKTEST " +str(n) + ": ", ret
print 

"""

complex description
#- rigid/medium/hard
#- enzyme/other
#  per chain:
#  - interface size
#  - surface size
#  - interface fraction

per chain: interface prediction
#- number of predictions
#- sensitivity
#- specificity
#- matthews correlation coefficient
#- significantly better than random (p-value from hypergeom dist)
#- significantly worse than random (p-value from hypergeom dist)

docking results:
-
for each run:
TAKE CARE OF SYMM
#- number of * generated in it0
#- number of ** generated in it0
#- number of *** generated in it0
#- number of * in top 400
#- number of ** in top 400
#- number of *** in top 400
#- number of * in it1
#- number of ** in it1
#- number of *** in it1
#- number of * in water
#- number of ** in water
#- number of *** in water
#- rank of the first * in it0
#- rank of the first ** in it0
#- rank of the first *** in it0
#- rank of the first * in it1
#- rank of the first ** in it1
#- rank of the first *** in it1
#- rank of the first * in water
#- rank of the first ** in water
#- rank of the first *** in water
#- rank of the first * in water, including fnat
#- rank of the first ** in water, including fnat
#- rank of the first *** in water, including fnat
#- number of * in top 400, significantly enriched (hypergeom)
#- number of * in top 400, significantly depleted (hypergeom)
#- number of ** in top 400, significantly enriched (hypergeom)
#- number of ** in top 400, significantly depleted (hypergeom)
#- number of clusters with at least * in top 4, including fnat
#- number of clusters with at least ** in top 4, including fnat
#- number of clusters with at least *** in top 4, including fnat
#- rank of the first cluster with at least * in top 4, including fnat
#- rank of the first cluster with at least ** in top 4, including fnat
#- rank of the first cluster with at least *** in top 4, including fnat
for run1:
#- average i-RMSD improvement + stdev: it0 => it1
#- average l-RMSD improvement + stdev: it0 => it1
#- average fnat improvement  + stdev: it0 => it1
#- average i-RMSD improvement + stdev: it1 => water
#- average l-RMSD improvement + stdev: it1 => water
#- average fnat improvement  + stdev: it1 => water

#- number of * in top 400, 
   run1 significantly better than run3 or other way around
   (fisher exact test)   
- Interface prediction with abundant contacts: it0, it1, water!

"""
