import os
from stattools import hypergeom

def get_stars_nofnat(itdir):
  l_rmsds = open(itdir+os.sep+"l-RMSD.dat").readlines()[1:]
  l_rmsds = [float(l.split()[1]) for l in l_rmsds]
  i_rmsds = open(itdir+os.sep+"i-RMSD.dat").readlines()[1:]
  i_rmsds = [float(l.split()[1]) for l in i_rmsds]
  if os.path.exists(itdir+os.sep+"sym-l-RMSD.dat"):
    sym_l_rmsds = open(itdir+os.sep+"sym-l-RMSD.dat").readlines()[1:]
    sym_l_rmsds = [float(l.split()[1]) for l in sym_l_rmsds]
    l_rmsds = [min(a,b) for a,b in zip(l_rmsds, sym_l_rmsds)]
    sym_i_rmsds = open(itdir+os.sep+"sym-i-RMSD.dat").readlines()[1:]
    sym_i_rmsds = [float(l.split()[1]) for l in sym_i_rmsds]
    i_rmsds = [min(a,b) for a,b in zip(i_rmsds, sym_i_rmsds)]
  stars = []
  for l_rmsd, i_rmsd in zip(l_rmsds, i_rmsds):
    if l_rmsd < 1 or i_rmsd < 1: stars.append(3)
    elif l_rmsd < 5 or i_rmsd < 2: stars.append(2)
    elif l_rmsd < 10 or i_rmsd < 4: stars.append(1)
    else: stars.append(0)
  return stars
  
def get_stars(itdir):
  stars0 = get_stars_nofnat(itdir)
  fnats = open(itdir+os.sep+"file.fnat").readlines()
  fnats = [float(l.split()[0]) for l in fnats]
  stars = []
  for fnat, star in zip(fnats, stars0):
    if fnat < 0.1: stars.append(0)
    elif fnat < 0.3: stars.append(min(star, 1))
    elif fnat < 0.5: stars.append(min(star, 2))
    else: stars.append(star)
  return stars      

def get_clusters(itdir):
  clusters = open(itdir+os.sep+"file.cluster").readlines()
  clusters = [int(l.split()[0]) for l in clusters]
  return clusters

def get_haddockscores(itdir):
  scores = open(itdir+os.sep+"file.list").readlines()
  scores = [float(l.split()[2]) for l in scores]
  return scores

def get_clusterstats(clusters, stats):
  clus0 = {}
  for c,s in zip(clusters, stats):
    if c == 0: continue
    if c not in clus0: clus0[c] = []
    clus0[c].append(s)
  nrclus = max(clus0.keys())
  clus = [clus0[n] for n in range(1,nrclus+1)]
  return clus

def runtest1(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Number of structures in it0: %s" % len(stars)
    
def runtest2(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Number of * (no fnat) generated in it0: %d" % (len([s for s in stars if s > 0]))

def runtest3(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Number of ** (no fnat) generated in it0: %d" % (len([s for s in stars if s >= 2]))

def runtest4(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Number of *** (no fnat) generated in it0: %d" % (len([s for s in stars if s == 3]))

def runtest5(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Number of * (no fnat) generated in it0 top 400: %d" % (len([s for s in stars[:400] if s > 0]))

def runtest6(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Number of ** (no fnat) generated in it0 top 400: %d" % (len([s for s in stars[:400] if s >= 2]))

def runtest7(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Number of *** (no fnat) generated in it0 top 400: %d" % (len([s for s in stars[:400] if s == 3]))

def runtest8(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s > 0])
  topstar = len([s for s in stars[:400] if s > 0])
  if totstar == 0: return 
  return "Enrichment factor for * (no fnat) in top 400: %.3f" % ((float(topstar)/400)/(float(totstar)/len(stars)))

def runtest9(protdir, prot, run):  
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s > 0])
  topstar = len([s for s in stars[:400] if s > 0])
  if totstar == 0: return 
  tp = topstar
  fp = 400 - topstar
  fn = totstar - topstar
  tn = len(stars) - tp - fp - fn
  pval = 0
  for n in range(0,min(fn+1,fp+1)):
    pval += hypergeom(tp+n, fp-n, tn+n, fn-n)      
  return "P-value of enrichment factor for * (no fnat) being better than random (hypergeom dist): %.3g" % pval

def runtest10(protdir, prot, run):  
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s > 0])
  topstar = len([s for s in stars[:400] if s > 0])
  if totstar == 0: return 
  tp = topstar
  fp = 400 - topstar
  fn = totstar - topstar
  tn = len(stars) - tp - fp - fn
  pval = 0
  for n in range(0,min(tp+1,tn+1)):
    pval += hypergeom(tp-n, fp+n, tn-n, fn+n)
  return "P-value of enrichment factor for * (no fnat) being worse than random (hypergeom dist): %.3g" % pval

def runtest11(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s >= 2])
  topstar = len([s for s in stars[:400] if s >= 2])
  if totstar == 0: return 
  return "Enrichment factor for ** (no fnat) in top 400: %.3f" % ((float(topstar)/400)/(float(totstar)/len(stars)))

def runtest12(protdir, prot, run):  
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s >= 2])
  topstar = len([s for s in stars[:400] if s >= 2])
  if totstar == 0: return 
  tp = topstar
  fp = 400 - topstar
  fn = totstar - topstar
  tn = len(stars) - tp - fp - fn
  pval = 0
  for n in range(0,min(fn+1,fp+1)):
    pval += hypergeom(tp+n, fp-n, tn+n, fn-n)      
  return "P-value of enrichment factor for ** (no fnat) being better than random (hypergeom dist): %.3g" % pval

def runtest13(protdir, prot, run):  
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s >= 2])
  topstar = len([s for s in stars[:400] if s >= 2])
  if totstar == 0: return 
  tp = topstar
  fp = 400 - topstar
  fn = totstar - topstar
  tn = len(stars) - tp - fp - fn
  pval = 0
  for n in range(0,min(tp+1,tn+1)):
    pval += hypergeom(tp-n, fp+n, tn-n, fn+n)
  return "P-value of enrichment factor for ** (no fnat) being worse than random (hypergeom dist): %.3g" % pval

def runtest14(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s == 3])
  topstar = len([s for s in stars[:400] if s == 3])
  if totstar == 0: return 
  return "Enrichment factor for *** (no fnat) in top 400: %.3f" % ((float(topstar)/400)/(float(totstar)/len(stars)))

def runtest15(protdir, prot, run):  
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s == 3])
  topstar = len([s for s in stars[:400] if s == 3])
  if totstar == 0: return 
  tp = topstar
  fp = 400 - topstar
  fn = totstar - topstar
  tn = len(stars) - tp - fp - fn
  pval = 0
  for n in range(0,min(fn+1,fp+1)):
    pval += hypergeom(tp+n, fp-n, tn+n, fn-n)      
  return "P-value of enrichment factor for ** (no fnat) being better than random (hypergeom dist): %.3g" % pval

def runtest16(protdir, prot, run):  
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  totstar = len([s for s in stars if s == 3])
  topstar = len([s for s in stars[:400] if s == 3])
  if totstar == 0: return 
  tp = topstar
  fp = 400 - topstar
  fn = totstar - topstar
  tn = len(stars) - tp - fp - fn
  pval = 0
  for n in range(0,min(tp+1,tn+1)):
    pval += hypergeom(tp-n, fp+n, tn-n, fn+n)
  return "P-value of enrichment factor for *** (no fnat) being worse than random (hypergeom dist): %.3g" % pval

def runtest17(protdir, prot, run):
  it1dir = os.sep.join((protdir, run, "it1"))
  if not os.path.exists(it1dir): return
  stars = get_stars_nofnat(it1dir)
  return "Number of * (no fnat) in it1: %d" % (len([s for s in stars if s > 0]))

def runtest18(protdir, prot, run):
  it1dir = os.sep.join((protdir, run, "it1"))
  if not os.path.exists(it1dir): return
  stars = get_stars_nofnat(it1dir)
  return "Number of ** (no fnat) in it1: %d" % (len([s for s in stars if s >= 2]))
  
  
def runtest19(protdir, prot, run):
  it1dir = os.sep.join((protdir, run, "it1"))
  if not os.path.exists(it1dir): return
  stars = get_stars_nofnat(it1dir)
  return "Number of *** (no fnat) in it1: %d" % (len([s for s in stars if s == 3]))

def runtest20(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars_nofnat(wdir)
  return "Number of * (no fnat) in water: %d" % (len([s for s in stars if s > 0]))

def runtest21(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars_nofnat(wdir)
  return "Number of ** (no fnat) in water: %d" % (len([s for s in stars if s >= 2]))

def runtest22(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars_nofnat(wdir)
  return "Number of *** (no fnat) in water: %d" % (len([s for s in stars if s == 3]))

def runtest23(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Rank of first * (no fnat) in it0: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] > 0][0])

def runtest24(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Rank of first ** (no fnat) in it0: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] >= 2][0])

def runtest25(protdir, prot, run):
  it0dir = os.sep.join((protdir, run, "it0"))
  stars = get_stars_nofnat(it0dir)
  return "Rank of first *** (no fnat) in it0: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] == 3][0])

def runtest26(protdir, prot, run):
  it1dir = os.sep.join((protdir, run, "it1"))
  if not os.path.exists(it1dir): return
  stars = get_stars_nofnat(it1dir)
  return "Rank of first * (no fnat) in it1: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] > 0][0])

def runtest27(protdir, prot, run):
  it1dir = os.sep.join((protdir, run, "it1"))
  if not os.path.exists(it1dir): return
  stars = get_stars_nofnat(it1dir)
  return "Rank of first ** (no fnat) in it1: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] >= 2][0])

def runtest28(protdir, prot, run):
  it1dir = os.sep.join((protdir, run, "it1"))
  if not os.path.exists(it1dir): return
  stars = get_stars_nofnat(it1dir)
  return "Rank of first *** (no fnat) in it1: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] == 3][0])

def runtest29(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars_nofnat(wdir)
  return "Rank of first * (no fnat) in water: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] > 0][0])

def runtest30(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars_nofnat(wdir)
  return "Rank of first ** (no fnat) in water: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] >= 2][0])

def runtest31(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars_nofnat(wdir)
  return "Rank of first *** (no fnat) in water: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] == 3][0])

def runtest32(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  return "Rank of first * (with fnat) in water: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] > 0][0])

def runtest33(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  return "Rank of first ** (with fnat) in water: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] >= 2][0])

def runtest34(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  return "Rank of first *** (with fnat) in water: %s" % ([s[1] for s in zip(stars,range(1,len(stars)+1))+[(3,"-")] if s[0] == 3][0])

def runtest35(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  clusters = get_clusters(wdir)
  ret = len(set([c for c in clusters if c]))
  return "Number of clusters: %d" %  ret

def runtest36(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  clusters = get_clusters(wdir)
  clusterstars = get_clusterstats(clusters, stars)
  clusterstars = [c[:4] for c in clusterstars]
  ret = len([c for c in clusterstars if max(c) > 0])
  return "Number of clusters with * (with fnat) in the top 4: %d" %  ret

def runtest37(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  clusters = get_clusters(wdir)
  clusterstars = get_clusterstats(clusters, stars)
  clusterstars = [c[:4] for c in clusterstars]
  ret = len([c for c in clusterstars if max(c) >= 2])
  return "Number of clusters with ** (with fnat) in the top 4: %d" %  ret

def runtest38(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  clusters = get_clusters(wdir)
  clusterstars = get_clusterstats(clusters, stars)
  clusterstars = [c[:4] for c in clusterstars]
  ret = len([c for c in clusterstars if max(c) == 3])
  return "Number of clusters with *** (with fnat) in the top 4: %d" %  ret

def runtest39(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  clusters = get_clusters(wdir)
  clusterstars = get_clusterstats(clusters, stars)
  scores = get_haddockscores(wdir)
  clusterscores = get_clusterstats(clusters, scores)
  clusterscores = [c[:4] for c in clusterscores]
  clusterscores = [sum(c)/len(c) for c in clusterscores]
  clusters = [(max(c[:4]), sc) for c,sc in zip(clusterstars, clusterscores)]
  clusters.sort(key=lambda a: a[1])
  ret = "-"
  c = [min(c[0], 1) for c in clusters]
  if 1 not in c: ret = "-"
  else: ret = c.index(1) + 1
  return "Rank of the first cluster with * (with fnat): %s" % ret 

def runtest40(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  clusters = get_clusters(wdir)
  clusterstars = get_clusterstats(clusters, stars)
  scores = get_haddockscores(wdir)
  clusterscores = get_clusterstats(clusters, scores)
  clusterscores = [c[:4] for c in clusterscores]
  clusterscores = [sum(c)/len(c) for c in clusterscores]
  clusters = [(max(c[:4]), sc) for c,sc in zip(clusterstars, clusterscores)]
  clusters.sort(key=lambda a: a[1])
  ret = "-"
  c = [min(c[0], 2) for c in clusters]
  if 2 not in c: ret = "-"
  else: ret = c.index(2) + 1
  return "Rank of the first cluster with ** (with fnat): %s" % ret 

def runtest41(protdir, prot, run):
  wdir = os.sep.join((protdir, run, "water"))
  if not os.path.exists(wdir): return
  stars = get_stars(wdir)
  clusters = get_clusters(wdir)
  clusterstars = get_clusterstats(clusters, stars)
  scores = get_haddockscores(wdir)
  clusterscores = get_clusterstats(clusters, scores)
  clusterscores = [c[:4] for c in clusterscores]
  clusterscores = [sum(c)/len(c) for c in clusterscores]
  clusters = [(max(c[:4]), sc) for c,sc in zip(clusterstars, clusterscores)]
  clusters.sort(key=lambda a: a[1])
  c = [min(c[0], 3) for c in clusters]
  if 3 not in c: ret = "-"
  else: ret = c.index(3) + 1
  return "Rank of the first cluster with *** (with fnat): %s" % ret 
