import os

from stattools import *

def get_l_rmsds(itdir):
  l_rmsds = open(itdir+os.sep+"l-RMSD.dat").readlines()[1:]
  l_rmsds = [float(l.split()[1]) for l in l_rmsds]
  if os.path.exists(itdir+os.sep+"sym-l-RMSD.dat"):
    sym_l_rmsds = open(itdir+os.sep+"sym-l-RMSD.dat").readlines()[1:]
    sym_l_rmsds = [float(l.split()[1]) for l in sym_l_rmsds]
    l_rmsds = [min(a,b) for a,b in zip(l_rmsds, sym_l_rmsds)]
  return l_rmsds

def get_i_rmsds(itdir):
  i_rmsds = open(itdir+os.sep+"i-RMSD.dat").readlines()[1:]
  i_rmsds = [float(l.split()[1]) for l in i_rmsds]
  if os.path.exists(itdir+os.sep+"sym-i-RMSD.dat"):
    sym_i_rmsds = open(itdir+os.sep+"sym-i-RMSD.dat").readlines()[1:]
    sym_i_rmsds = [float(l.split()[1]) for l in sym_i_rmsds]
    i_rmsds = [min(a,b) for a,b in zip(i_rmsds, sym_i_rmsds)]
  return i_rmsds

def get_fnats(itdir):
  fnats = open(itdir+os.sep+"file.fnat").readlines()
  fnats = [float(l.split()[0]) for l in fnats]
  return fnats

def get_mapping(itdir):
  numbers =  open(itdir+os.sep+"file.nam").readlines()
  numbers = [n.strip()[:-len(".pdb")] for n in numbers]
  if numbers[0].endswith("w"):
    numbers = [n[:-1] for n in numbers]
  numbers = [int(n[n.rindex("_")+1:]) for n in numbers]
  return numbers
  
def docktest1(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  rmsd_it0 = get_l_rmsds(it0dir)
  rmsd_it1 = get_l_rmsds(it1dir)
  mapping = get_mapping(it1dir)
  rmsd_it1 = dict(zip(mapping, rmsd_it1))
  rmsd_it1 = [rmsd_it1[n] for n in sorted(mapping)]
  rmsd_diff = [post - pre for post, pre in zip(rmsd_it1, rmsd_it0)]
  return "run1 l-RMSD shift from it0 to it1: %.3f +/- %.3f" % (average(rmsd_diff))

def docktest2(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  rmsd_it0 = get_i_rmsds(it0dir)
  rmsd_it1 = get_i_rmsds(it1dir)
  mapping = get_mapping(it1dir)
  rmsd_it1 = dict(zip(mapping, rmsd_it1))
  rmsd_it1 = [rmsd_it1[n] for n in sorted(mapping)]
  rmsd_diff = [post - pre for post, pre in zip(rmsd_it1, rmsd_it0)]
  return "run1 i-RMSD shift from it0 to it1: %.3f +/- %.3f" % (average(rmsd_diff))

def docktest3(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  fnat_it0 = get_fnats(it0dir)
  fnat_it1 = get_fnats(it1dir)
  mapping = get_mapping(it1dir)
  fnat_it1 = dict(zip(mapping, fnat_it1))
  fnat_it1 = [fnat_it1[n] for n in sorted(mapping)]
  fnat_diff = [post - pre for post, pre in zip(fnat_it1, fnat_it0)]
  return "run1 fnat shift from it0 to it1: %.3f +/- %.3f" % (average(fnat_diff))

def docktest4(protdir, prot):
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  waterdir = os.sep.join((protdir, "run1", "water"))
  rmsd_it1 = get_l_rmsds(it1dir)
  rmsd_w = get_l_rmsds(waterdir)
  mapping = get_mapping(it1dir)
  rmsd_it1 = dict(zip(mapping, rmsd_it1))
  rmsd_it1 = [rmsd_it1[n] for n in sorted(mapping)]
  mapping = get_mapping(waterdir)
  rmsd_w = dict(zip(mapping, rmsd_w))
  rmsd_w = [rmsd_w[n] for n in sorted(mapping)]  
  rmsd_diff = [post - pre for post, pre in zip(rmsd_w, rmsd_it1)]
  return "run1 l-RMSD shift from it1 to water: %.3f +/- %.3f" % (average(rmsd_diff))

def docktest5(protdir, prot):
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  waterdir = os.sep.join((protdir, "run1", "water"))
  rmsd_it1 = get_i_rmsds(it1dir)
  rmsd_w = get_i_rmsds(waterdir)
  mapping = get_mapping(it1dir)
  rmsd_it1 = dict(zip(mapping, rmsd_it1))
  rmsd_it1 = [rmsd_it1[n] for n in sorted(mapping)]
  mapping = get_mapping(waterdir)
  rmsd_w = dict(zip(mapping, rmsd_w))
  rmsd_w = [rmsd_w[n] for n in sorted(mapping)]  
  rmsd_diff = [post - pre for post, pre in zip(rmsd_w, rmsd_it1)]
  return "run1 i-RMSD shift from it1 to water: %.3f +/- %.3f" % (average(rmsd_diff))

def docktest6(protdir, prot):
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  waterdir = os.sep.join((protdir, "run1", "water"))
  fnat_it1 = get_fnats(it1dir)
  fnat_w = get_fnats(waterdir)
  mapping = get_mapping(it1dir)
  fnat_it1 = dict(zip(mapping, fnat_it1))
  fnat_it1 = [fnat_it1[n] for n in sorted(mapping)]
  mapping = get_mapping(waterdir)
  fnat_w = dict(zip(mapping, fnat_w))
  fnat_w = [fnat_w[n] for n in sorted(mapping)]  
  fnat_diff = [post - pre for post, pre in zip(fnat_w, fnat_it1)]
  return "run1 fnat shift from it1 to water: %.3f +/- %.3f" % (average(fnat_diff))

def docktest7(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  rmsd_it0 = get_l_rmsds(it0dir)
  rmsd_it1 = get_l_rmsds(it1dir)
  mapping = get_mapping(it1dir)
  rmsd_it1 = dict(zip(mapping, rmsd_it1))
  rmsd_it1 = [rmsd_it1[n] for n in sorted(mapping)]
  i_rmsd_it0 = get_i_rmsds(it0dir)
  rmsd_diff = [post - pre for post, pre, irmsd in zip(rmsd_it1, rmsd_it0, i_rmsd_it0) if irmsd <= 4]
  if len(rmsd_diff) == 0 :return
  return "run1 l-RMSD shift from it0 to it1, for structures with i-RMSD < 4 in it0: %.3f +/- %.3f" % (average(rmsd_diff))

def docktest8(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  rmsd_it0 = get_i_rmsds(it0dir)
  rmsd_it1 = get_i_rmsds(it1dir)
  mapping = get_mapping(it1dir)
  rmsd_it1 = dict(zip(mapping, rmsd_it1))
  rmsd_it1 = [rmsd_it1[n] for n in sorted(mapping)]
  i_rmsd_it0 = get_i_rmsds(it0dir)
  rmsd_diff = [post - pre for post, pre, irmsd in zip(rmsd_it1, rmsd_it0, i_rmsd_it0) if irmsd <= 4]
  if len(rmsd_diff) == 0 :return
  return "run1 i-RMSD shift from it0 to it1, for structures with i-RMSD < 4 in it0: %.3f +/- %.3f" % (average(rmsd_diff))

def docktest9(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  fnat_it0 = get_fnats(it0dir)
  fnat_it1 = get_fnats(it1dir)
  mapping = get_mapping(it1dir)
  fnat_it1 = dict(zip(mapping, fnat_it1))
  fnat_it1 = [fnat_it1[n] for n in sorted(mapping)]
  i_rmsd_it0 = get_i_rmsds(it0dir)
  fnat_diff = [post - pre for post, pre, irmsd in zip(fnat_it1, fnat_it0, i_rmsd_it0) if irmsd <= 4]
  if len(fnat_diff) == 0 :return
  return "run1 fnat shift from it0 to it1, for structures with i-RMSD < 4 in it0: %.3f +/- %.3f" % (average(fnat_diff))

def docktest10(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  waterdir = os.sep.join((protdir, "run1", "water"))
  rmsd_it1 = get_l_rmsds(it1dir)
  rmsd_w = get_l_rmsds(waterdir)
  mapping = get_mapping(it1dir)
  rmsd_it1 = dict(zip(mapping, rmsd_it1))
  rmsd_it1 = [rmsd_it1[n] for n in sorted(mapping)]
  mapping = get_mapping(waterdir)
  rmsd_w = dict(zip(mapping, rmsd_w))
  rmsd_w = [rmsd_w[n] for n in sorted(mapping)]  
  i_rmsd_it0 = get_i_rmsds(it0dir)
  rmsd_diff = [post - pre for post, pre, irmsd in zip(rmsd_w, rmsd_it1, i_rmsd_it0) if irmsd <= 4]
  if len(rmsd_diff) == 0 :return
  return "run1 l-RMSD shift from it1 to water, for structures with i-RMSD < 4 in it0: %.3f +/- %.3f" % (average(rmsd_diff))

def docktest11(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  waterdir = os.sep.join((protdir, "run1", "water"))
  rmsd_it1 = get_i_rmsds(it1dir)
  rmsd_w = get_i_rmsds(waterdir)
  mapping = get_mapping(it1dir)
  rmsd_it1 = dict(zip(mapping, rmsd_it1))
  rmsd_it1 = [rmsd_it1[n] for n in sorted(mapping)]
  mapping = get_mapping(waterdir)
  rmsd_w = dict(zip(mapping, rmsd_w))
  rmsd_w = [rmsd_w[n] for n in sorted(mapping)]  
  i_rmsd_it0 = get_i_rmsds(it0dir)
  rmsd_diff = [post - pre for post, pre, irmsd in zip(rmsd_w, rmsd_it1, i_rmsd_it0) if irmsd <= 4]
  if len(rmsd_diff) == 0 :return
  return "run1 i-RMSD shift from it1 to water, for structures with i-RMSD < 4 in it0: %.3f +/- %.3f" % (average(rmsd_diff))

def docktest12(protdir, prot):
  it0dir = os.sep.join((protdir, "run1", "it0"))
  it1dir = os.sep.join((protdir, "run1", "it1"))
  if not os.path.exists(it1dir): return
  waterdir = os.sep.join((protdir, "run1", "water"))
  fnat_it1 = get_fnats(it1dir)
  fnat_w = get_fnats(waterdir)
  mapping = get_mapping(it1dir)
  fnat_it1 = dict(zip(mapping, fnat_it1))
  fnat_it1 = [fnat_it1[n] for n in sorted(mapping)]
  mapping = get_mapping(waterdir)
  fnat_w = dict(zip(mapping, fnat_w))
  fnat_w = [fnat_w[n] for n in sorted(mapping)]  
  i_rmsd_it0 = get_i_rmsds(it0dir)
  fnat_diff = [post - pre for post, pre, irmsd in zip(fnat_w, fnat_it1, i_rmsd_it0) if irmsd <= 4]
  if len(fnat_diff) == 0 :return
  return "run1 fnat shift from it1 to water, for structures with i-RMSD < 4 in it0: %.3f +/- %.3f" % (average(fnat_diff))

def docktest13(protdir, prot):
  run1dir = os.sep.join((protdir, "run1", "it0"))
  run3dir = os.sep.join((protdir, "run3", "it0"))
  from test2 import get_stars_nofnat
  from stattools import fishex
  run1stars = get_stars_nofnat(run1dir) 
  run1s = len([c for c in  run1stars if c > 0])
  run3stars = get_stars_nofnat(run3dir) 
  run3s = len([c for c in  run3stars if c > 0])

  pval = 0
  for n in range(0,min(run3s+1,len(run1stars)-run1s+1)):
    pval += fishex(run1s+n, len(run1stars), run3s-n, len(run3stars))
  return "P-value that run1 is better than run3 (Fisher exact test): %.3f" % pval

def docktest14(protdir, prot):
  run1dir = os.sep.join((protdir, "run1", "it0"))
  run3dir = os.sep.join((protdir, "run3", "it0"))
  from test2 import get_stars_nofnat
  from stattools import fishex
  run1stars = get_stars_nofnat(run1dir) 
  run1s = len([c for c in  run1stars if c > 0])
  run3stars = get_stars_nofnat(run3dir) 
  run3s = len([c for c in  run3stars if c > 0])

  pval = 0
  for n in range(0,min(run1s+1,len(run3stars)-run3s+1)):
    pval += fishex(run3s+n, len(run3stars), run1s-n, len(run1stars))
  return "P-value that run1 is worse than run3 (Fisher exact test): %.3f" % pval
